
#include "ch.h"
#include "hal.h"

#if HAL_USE_DDS

#define DDS_FCLK            ((1ULL << 59) / STM32_SYSCLK)   /* 2^28 * 2^31 / DDS_CLOCK */
#define DDS_WAKEUP_DELAY    10                              /* 10 ms delay for DDS wakeup from start */

/* SPI Configuration: SYSCLK / 2 (16 MHZ), 16 bit MSB;
   PB12 - NSS, PB13 - SCK, PB14 - MISO (not used), PB15 - MOSI
*/
static const SPIConfig spicfg = {
  NULL, GPIOB, DDS_CS, SPI_CR1_CPOL | SPI_CR1_DFF
};

static unsigned int freq_idx;   /* FREQ0/FREQ1 switcher */

void ddsInit(void) {
/* AD9834 DDS initialization */
    uint16_t reg;

    spiAcquireBus(&SPID2);
    spiStart(&SPID2, &spicfg);
    spiUnselect(&SPID2);
    chThdSleepMilliseconds(DDS_WAKEUP_DELAY);

    /* NOTE: Clock output is defined in mcuconf.h */

    /* Perform reset */
    reg = 0x2100;
    spiSelect(&SPID2);
    spiSend(&SPID2, 1, &reg);
    spiUnselect(&SPID2);
    chThdSleepMilliseconds(DDS_WAKEUP_DELAY);
    
    /* Remove reset */
    reg = 0x2000;
    spiSelect(&SPID2);
    spiSend(&SPID2, 1, &reg);
    spiUnselect(&SPID2);
    spiStop(&SPID2);
    spiReleaseBus(&SPID2);
    
    freq_idx = 0;
}

void ddsTune(const uint32_t freq) {
/* Tune to a given frequency */
    uint32_t ftw;
    uint16_t buf[3];

    /* Calculate FTW */
    ftw = (uint32_t)(freq * DDS_FCLK >> 31);
	ftw = (((ftw & 0xffffff00) << 2) & 0x3fff0000) | (ftw & 0x00003fff);

	/* FREQ0/FREQ1 switch */
	if (freq_idx) {
		ftw |= 0x40004000;
		freq_idx = 0;
		buf[2] = 0x2000;
	} else {
		ftw |= 0x80008000;
		freq_idx = 1;
		buf[2] = 0x2800;
	}
	buf[0] = (uint16_t)ftw;			/* FREQ0 (bytes 0,1) */
	buf[1] = (uint16_t)(ftw >> 16);	/* FREQ0 (bytes 2,3) */

	/* Send FTW to DDS */
    spiAcquireBus(&SPID2);
    spiStart(&SPID2, &spicfg);
    spiSelect(&SPID2);
    spiSend(&SPID2, 3, buf);
    spiUnselect(&SPID2);
    spiStop(&SPID2);
    spiReleaseBus(&SPID2);
}

#endif /* HAL_USE_DDS */
